from django.test import TestCase, RequestFactory

from views import StaticSiteTemplateView


class TestURLNormalisation(TestCase):

    def test_empty_path_results_in_index_html(self):
        kwargs = {
            'url':"",
        }

        theView = StaticSiteTemplateView(kwargs=kwargs)

        normalised_url = theView.normalised_url()

        self.assertEquals('index.html', normalised_url)

    def test_directory_index_is_correctly_expanded(self):
        kwargs = {
            'url':"directory/",
        }

        theView = StaticSiteTemplateView(kwargs=kwargs)

        normalised_url = theView.normalised_url()

        self.assertEquals('directory/index.html', normalised_url)

    def test_full_paths_are_unaltered(self):
        kwargs = {
            'url':"directory/subdirectory/real_file.html",
        }

        theView = StaticSiteTemplateView(kwargs=kwargs)

        normalised_url = theView.normalised_url()

        self.assertEquals('directory/subdirectory/real_file.html', normalised_url)



# -*- coding: utf-8 -*-

from django.views.generic import TemplateView
from django.http import Http404
from django.template import loader, TemplateDoesNotExist
from django.conf import settings
from django.utils.importlib import import_module

def get_context_map(settings):
    try:
        helper_module_name = settings.STATICSITE_CONTEXT_HELPER_MODULE
    except AttributeError:
        return {}

    # Failure beyond here is a configuration error.
    # - If you define a helper module it must exist and expose the context_map attribute.
    helper_module = import_module(helper_module_name)
    return helper_module.context_map

CONTEXT_MAP = get_context_map(settings)

DEFAULT_INDEX = 'index.html'
STATICSITE_TEMPLATE_DIR = 'staticsite/'

#### Notes
# 1. Currently breaking APPEND_SLASHES. So DEBUG only. (which is probably fine)
# 2. If we can solve 1, then WRITE_TO_DIR with a Warning if the file already exists.
#       — i.e. the idea is to have the web server serve the file.
#       Writing it once is OK. Writing it again-and-again implies a
#       configuration error.
# 3. How do I clean the URL patterns so I can just include('staticsite.urls') ?

class StaticSiteTemplateView(TemplateView):

    def normalised_url(self):
        the_url = self.kwargs['url']

        if the_url == '': the_url = DEFAULT_INDEX
        if the_url.endswith('/'): the_url += DEFAULT_INDEX

        return the_url

    def get_template_names(self):
        template_name = STATICSITE_TEMPLATE_DIR + self.normalised_url()

        # http://stackoverflow.com/questions/5690213/how-to-check-if-a-template-exists-in-django
        #
        # Q: is there a better way of handling this by subclassing
        # TemplateResponse, and assigning that as the view's
        # response_class?
        # <https://docs.djangoproject.com/en/1.5/ref/template-response/#django.template.response.TemplateResponse>
        try:
            loader.get_template(template_name)
        except TemplateDoesNotExist:
            raise Http404

        return [template_name,]

    def get_context_data(self, **kwargs):
        context = super(StaticSiteTemplateView, self).get_context_data(**kwargs)

        if CONTEXT_MAP.has_key(self.normalised_url()):
            context.update(CONTEXT_MAP[self.normalised_url()])

        return context



# -*- coding: utf-8 -*-
import os
import codecs

from django.core.management.base import BaseCommand, CommandError
from django.conf import settings
from django.template import Template, Context
from django.template.loader import get_template

### Notes
#   Expects `staticsite` folder to exist in `settings.TEMPLATE_DIRS`
#       This COULD use Template Loaders.
#
#   Uses `settings.STATICSITE_OUTPUT_DIR` for write location.
#       MAYBE make this a command line argument.
#
#   TODO: Set STATIC_URL context variable from settings.
#   TODO: Render the view here. (Take advantage of get_context_data)
#   TODO: Cache templates so we only write changed files.
#           - c.p. django.contrib.staticfiles collectstatic
class Command(BaseCommand):
    help = 'Writes (HTML) files for templates in staticsite template dir.'

    def handle(self, *args, **options):
        staticsite_dir = self._staticsite_dir()
        output_dir = settings.STATICSITE_OUTPUT_DIR

        for dirpath, subdirs, filenames in os.walk(staticsite_dir):
                for name in filenames:
                    if name.startswith('.'):
                        continue # Put this in for OS X's .DS_Store files.
                                 # TODO: Think about this.

                    # get template and render
                    template_path = os.path.join(dirpath, name)
                    print "Loading template at: %s" % template_path

                    t = get_template(template_path)
                    html = t.render(Context({'STATIC_URL':'/static/'}))

                    # and write
                    write_dir = dirpath.replace(staticsite_dir, output_dir, 1)
                    if not os.path.exists(write_dir):
                        os.makedirs(write_dir)
                    write_path = os.path.join(write_dir, name)

                    write_file = codecs.open(write_path, encoding='utf-8', mode='w')
                    write_file.write(html)
                    write_file.close()

                    print "Wrote: %s" % write_path


    def _staticsite_dir(self):
        for template_dir in settings.TEMPLATE_DIRS:
            for path in os.listdir(template_dir):
                if path == 'staticsite':
                    staticsite_dir = os.path.join(template_dir, path)
                    self.stdout.write('Building staticsite from %s' % staticsite_dir)
                    return staticsite_dir
        raise CommandError('staticsite dir not found in settings.TEMPLATE_DIRS')
